@@ -1,5 +1,4 @@
 # GNU Solfege - free ear training software
-
 # vim: set fileencoding=utf-8 :
 # Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Tom Cato Amundsen
 #
@@ -42,6 +41,7 @@ from solfege import utils
 from solfege import uuid
 from solfege import xrandom
 from solfege.dataparser import istr
+from solfege.mpd import elems
 from solfege.mpd import mpdutils
 from solfege.mpd.duration import Duration
 from solfege.mpd.rat import Rat
@@ -50,7 +50,7 @@ _test_mode = False
 
 # The name of the folder containing the standard learning trees.
 # Relative to the installation dir.
-exercises_dir = u"exercises/standard"
+exercises_dir = os.path.join(u"exercises", u"standard")
 _abs_exercises_dir = os.path.join(os.getcwdu(), exercises_dir) + os.sep
 
 solfege_uri = u"solfege:"
@@ -284,11 +284,11 @@ def rnc_markup_tokenizer(s):
     """
     [rn][mod1][num][\s-]
     """
-    rn_re = re.compile("""(?P<p1>b?[ivIV]+)
+    rn_re = re.compile(u"""(?P<p1>[b♭♯#]?[ivIV]+)
                           (?P<p2>[^\d\s-]*)
                           (?P<p3>[^\s-]*)
                           (?P<sep>(\s*-\s*|\s*))""",
-                       re.VERBOSE)
+                       re.VERBOSE|re.UNICODE)
     i = 0
     retval = []
     while i < len(s):
@@ -377,111 +377,6 @@ class _Header(dict):
             return self[name]
         return istr("")
 
-class Bar(list):
-    class Item(object):
-        """
-        An object representing either a note or a rest.
-        """
-        def __init__(self, isnote, duration, tie):
-            assert isinstance(duration, Duration)
-            self.m_isnote = isnote
-            self.m_duration = duration
-            self.m_tie = tie
-    class BadDuration(Exception):
-        pass
-    class FullException(Exception):
-        pass
-    stepwidth = 30
-    def __init__(self, keysig):
-        assert isinstance(keysig, Rat)
-        self.m_keysig = keysig
-    def _do_add(self, isnote, n):
-        if not isinstance(n, Duration):
-            if not isinstance(n, unicode):
-                raise self.BadDuration(n)
-            n = Duration.new_from_string(n)
-        if self.duration() + n.get_rat_value() > self.m_keysig:
-            raise self.FullException()
-        self.append(self.Item(isnote, n, False))
-    def isfull(self):
-        return self.duration() == self.m_keysig
-    def can_append(self, item):
-        """
-        Return True if the bar is not full and there is space to add item.
-        """
-        return (self.duration()
-                + item.m_duration.get_rat_value() <= self.m_keysig)
-    def add_note(self, n):
-        self._do_add(True, n)
-    def add_rest(self, n):
-        self._do_add(False, n)
-    def duration(self):
-        r = Rat(0)
-        for item in self:
-            r += item.m_duration.get_rat_value()
-        return r
-    def do_layout(self, show_timesig):
-        self.m_show_timesig = show_timesig
-        if show_timesig:
-            xpos = self.stepwidth
-        else:
-            xpos = self.stepwidth / 2
-        for i in self:
-            i.m_xpos = xpos
-            xpos += 30
-        #If the bar is full, then the width of the bar is the m_xpos
-        #of the last note + Bar.stepwidth
-        if self.isfull():
-            self.m_width = self[-1].m_xpos + Bar.stepwidth
-        self.m_width = xpos + int((self.m_keysig - self.duration()) * 4 * Bar.stepwidth)
-    def _get_timelist(self):
-        """
-        A timelist is a list of list:
-        [isnote, duration] of the types [bool, Rat]
-        """
-        v = []
-        tie = False
-        for item in self:
-            if tie:
-                v[-1][1] += item.m_duration.get_rat_value()
-            else:
-                v.append([item.m_isnote, item.m_duration.get_rat_value()])
-            tie = item.m_tie
-        return v
-
-
-class RhythmStaff(list):
-    """
-    A staff is a list of Bars
-    A object that know how long rhythm to enter, and will
-    know of all time signature changes and bar lines.
-
-    A question in the lesson file could be written like this:
-    \time 3/4 c4 c8 c8 c4 | \time 3/8 c8 c8 c16 c16
-    or:
-    4 8 8 4 | 8 8 16 16
-
-    We need code that can transform the former to the last format so
-    that we can compare the real music to the rhythm entered by
-    the user.
-    """
-    def do_layout(self):
-        timesig = None
-        xpos = 0
-        for bar in self:
-            bar.m_xpos = xpos
-            bar.do_layout(timesig != bar.m_keysig)
-            timesig = bar.m_keysig
-            xpos += bar.m_width
-    def get_timelist(self):
-        v = []
-        for bar in self:
-            v.extend(bar._get_timelist())
-        return v
-    def append(self, bar):
-        if not isinstance(bar, Bar):
-            raise TypeError("RhythmStaff.append: only Bar allowed")
-        list.append(self, bar)
 
 class MusicBaseClass(object):
     """
@@ -573,40 +468,19 @@ class MpdParsable(MusicBaseClass):
             # added music code, not in the m_musicdata code.
             return
         exception.m_obj_lineno = exception.m_lineno - 1
-    def get_rhythmstaff(self, lessonfile_ref):
+    def get_score(self, lessonfile_ref, as_name=None):
         """
-        Create and return a RhythmStaff showing the rhythm of the music
-        object.
+        Return a elems.Score object of the music object. Report the variable
+        named by the as_name variable as the music object variable with the
+        bug if a MpdException is raised.
         """
-        staff = RhythmStaff()
-        timesig = None
-        bar_content = Rat(0, 1)
-        # the current note/rest-duration
-        duration = Duration(4, 0)
-        for toc_type, toc in mpd.parser.Lexer(
-                self.get_mpd_music_string(lessonfile_ref)):
-            if toc_type in (mpd.parser.Lexer.NOTE, mpd.parser.Lexer.REST):
-                if timesig is None:
-                    # very first note and no timesig parsed
-                    staff.append(Bar(Rat(4, 4)))
-                    timesig = Rat(4, 4)
-                if staff[-1].isfull():
-                    staff.append(Bar(timesig))
-                if toc.m_duration:
-                    duration = toc.m_duration
-                staff[-1].append(Bar.Item(toc_type == mpd.parser.Lexer.NOTE,
-                                duration, False))
-            elif toc_type == mpd.parser.Lexer.TIME:
-                timesig = Rat(toc.m_num, toc.m_den)
-                staff.append(Bar(timesig))
-            elif toc_type == u'~':
-                staff[-1][-1].m_tie = True
-            elif toc_type == u'|':
-                if staff[-1].m_keysig != bar_content:
-                    print "FIXME: barlines not ok."
-        return staff
-            
- 
+        try:
+            return mpd.parser.parse_to_score_object(self.get_mpd_music_string(lessonfile_ref))
+        except mpd.MpdException, e:
+            if as_name:
+                e.m_mpd_varname = as_name
+            raise
+
 
 class MpdDisplayable(MpdParsable):
     pass
@@ -621,12 +495,12 @@ class MpdTransposable(MpdDisplayable):
         # We will display the first 3 non-empty lines
         v = [x.strip() for x in self.m_musicdata.split("\n") if x]
         v = v[:3]
-        raise MusicObjectException("%s\n%s" % 
+        raise MusicObjectException("%s\n%s" %
             ("get_first_pitch() could not find a musical pitch in the music data:",
              "\n".join(v)))
     def get_musicdata_transposed(self, lessonfile_ref):
         """
-        Return m_musicdata, but transposed relative to 
+        Return m_musicdata, but transposed relative to
         lesonfile_ref.m_transpose. All pitches identified as Lexer.NOTE
         are transposed.
         """
@@ -844,7 +718,7 @@ class Rvoice(VoiceCommon):
             # no translation
             return "\\staff\\relative %s{\n%s\n}" \
               % (errpitch, lex.m_string)
-            
+
         pitch.m_octave_i = 0
         lex.set_first_pitch(pitch)
         if lessonfile_ref.header.random_transpose[0]:
@@ -952,31 +826,36 @@ class Satb(ChordCommon):
 
 class PercBaseClass(MpdParsable):
     def get_mpd_music_string(self, lessonfile_ref):
-        return "\\staff{\n%s\n}" % self.m_musicdata
+        return "\\rhythmstaff{\n%s\n}" % self.m_musicdata
     def _gen_track(self, lessonfile_ref, question):
         try:
             score = mpd.parser.parse_to_score_object(self.get_mpd_music_string(lessonfile_ref))
         except mpd.MpdException, e:
             self.complete_to_musicdata_coords(lessonfile_ref, e)
             raise
-        track = score.get_midi_events_as_percussion()[0]
+        track = mpd.score_to_tracks(score)[0]
         track.set_volume(cfg.get_int('config/preferred_instrument_volume'))
         track.prepend_bpm(lessonfile_ref.get_tempo()[0],
                           lessonfile_ref.get_tempo()[1])
         return track
+    def play(self, lessonfile_ref, question):
+        soundcard.synth.play_track(self._gen_track(lessonfile_ref, question))
 
 
 class Rhythm(PercBaseClass):
-    def play(self, lessonfile_ref, question):
-        track = self._gen_track(lessonfile_ref, question)
-        track.replace_note(mpd.notename_to_int("c"), 37)
-        track.replace_note(mpd.notename_to_int("d"), 80)
-        soundcard.synth.play_track(track)
+    def get_mpd_music_string(self, lessonfile_ref):
+        v = list(mpd.parser.Lexer(self.m_musicdata))
+        for idx, (toc_type, toc) in enumerate(v):
+            if toc_type == mpd.parser.Lexer.NOTE:
+                if toc.m_pitch.semitone_pitch() == 48:
+                    toc.m_pitch.set_from_int(cfg.get_int("config/rhythm_perc"))
+                if toc.m_pitch.semitone_pitch() == 50:
+                    toc.m_pitch.set_from_int(cfg.get_int("config/countin_perc"))
+        return "\\rhythmstaff{\n%s\n}" % mpd.parser.Lexer.to_string(v)
 
 
 class Percussion(PercBaseClass):
-    def play(self, lessonfile_ref, question):
-        soundcard.synth.play_track(self._gen_track(lessonfile_ref, question))
+    pass
 
 class _MusicExternalPlayer(MusicBaseClass):
     def __init__(self, typeid, musicdata):
@@ -1392,6 +1271,15 @@ class QuestionsLessonfile(LessonfileCommon):
         header.random_transpose is set.
         """
         return " ".join(self.get_music_as_notename_list(varname))
+    def get_score(self, varname='music'):
+        """
+        Return the elems.Score object of the music in the variable named
+        by VARNAME. 
+        """
+        assert self._idx is not None
+        musicobj = self.m_questions[self._idx][varname]
+        assert isinstance(musicobj, MpdParsable)
+        return musicobj.get_score(self, as_name=varname)
     def has_question(self):
         """
         Return True if a question is selected.
@@ -1450,7 +1338,7 @@ class QuestionsLessonfile(LessonfileCommon):
         if we select "Override default instrument" should call this method.
         """
         assert count in (1, 2, 3)
-        if (count in (2, 3) and 
+        if (count in (2, 3) and
             cfg.get_bool('config/override_default_instrument')):
                 retval = [cfg.get_int('config/lowest_instrument'),
                           cfg.get_int('config/lowest_instrument_volume'),
@@ -1585,6 +1473,73 @@ class DictationLessonfile(QuestionsLessonfile):
 class SingChordLessonfile(QuestionsLessonfile):
     pass
 
+
+class RhythmDictation2Lessonfile(QuestionsLessonfile):
+    """
+    We inherit from QuestionsLessonfile just to get the .get_tempo()
+    method. Lots of the other methods from QuestionsLessonfile does
+    not make sense to use in the class.
+    """
+    def parse_string(self, s, really_filename=None):
+        LessonfileCommon.parse_string(self, s, really_filename)
+        if not self.m_questions:
+            raise NoQuestionsInFileException(self.m_filename)
+        # Make sure the variables are lists. If there is only one elemt
+        # in the lesson file then we must make a list of them.
+        for question in self.m_questions:
+            if not isinstance(question['bars'], list):
+                question['bars'] = [question['bars']]
+            if not isinstance(question['elements'], list):
+                question['elements'] = [question['elements']]
+    def generate_random_question(self):
+        def rat_len_of_digits(digits):
+            """
+            Return a Rat representing the length of the digits.
+            "4 8 8" returns Rat(2, 4)
+            """
+            ret = Rat(0, 1)
+            for d in digits.split():
+                ret += Duration.new_from_string(d).get_rat_value()
+            return ret
+        notename = mpd.MusicalPitch.new_from_int(cfg.get_int("config/rhythm_perc")).get_octave_notename()
+        self._idx = random.randint(0, len(self.m_questions) - 1)
+        # The score where the user enters his answer
+        self.m_answer_score = score = elems.Score()
+        score.add_staff(staff_class=elems.RhythmStaff)
+        for num, den in self.m_questions[self._idx]['bars']:
+            score.add_bar(elems.TimeSignature(num, den))
+        for bar in score.m_bars:
+            bar.fill_skips(score.voice11)
+        # the question being played
+        self.m_question_score = score = elems.Score()
+        score.add_staff(staff_class=elems.RhythmStaff)
+        for num, den in self.m_questions[self._idx]['bars']:
+            score.add_bar(elems.TimeSignature(num, den))
+            bar = self.m_question_score.m_bars[-1]
+            total = Rat(0, 1)
+            # We try only 1000 times, since it is possible that the lesson
+            # file is badly written with too long elements or too short bars.
+            count = 0
+            max_count = 100
+            while total < bar.m_timesig.as_rat() and count < max_count:
+                elem = random.choice(self.m_questions[self._idx]['elements'])
+                if total + rat_len_of_digits(elem) > bar.m_timesig.as_rat():
+                    count += 1
+                    continue
+                total += rat_len_of_digits(elem)
+                for e in elem.split(" "):
+                    e = e.strip()
+                    n = elems.Note.new_from_string(u"%s%s" % (notename, e))
+                    score.voice11.append(n)
+                    count = 0
+                count += 1
+            if count == max_count:
+                raise LessonfileException("Bad elements variable in the lessonfile. Too long elements.")
+    def play_question(self):
+        tracks = mpd.score_to_tracks(self.m_question_score)
+        tracks[0].prepend_bpm(*self.m_questions[self._idx]['tempo'])
+        soundcard.synth.play_track(*tracks)
+
 class NameIntervalLessonfile(HeaderLessonfile):
     def parse_string(self, s, really_filename=None):
         super(NameIntervalLessonfile, self).parse_string(s, really_filename)
@@ -1805,7 +1760,7 @@ class InfoCache(object):
                 self.parse_file(filename)
             return self._data[filename][field]
         def parse_file(self, filename):
-            from solfege import frontpage 
+            from solfege import frontpage
             try:
                 p = frontpage.load_tree(filename)
                 self._data[filename] = {'title': p.m_name,
@@ -1831,19 +1786,22 @@ class InfoCache(object):
             self.parse_file(filename, mtime)
         return self._data[filename][field]
     def parse_file(self, filename, mtime=None):
-        logging.debug("parse_file %s" % filename)
         assert is_uri(filename) or os.path.isabs(filename)
         p = parse_lesson_file_header(uri_expand(filename))
         if not p:
             raise self.FileNotLessonfile(filename)
-        self._data[filename] = {
-            'title': p.header.get('title', 'error: no title in file'),
-            'module': p.header['module'],
-            'test': p.header.get('test', None),
-            'test_requirement': p.header.get('test_requirement', None),
-            'mtime': mtime if mtime else os.path.getmtime(uri_expand(filename)),
-            'replaces': p.header.get('replaces', []),
+        try:
+            self._data[filename] = {
+                'title': p.header.get('title', 'error: no title in file'),
+                'module': p.header['module'],
+                'test': p.header.get('test', None),
+                'test_requirement': p.header.get('test_requirement', None),
+                'mtime': mtime if mtime else os.path.getmtime(uri_expand(filename)),
+                'replaces': p.header.get('replaces', []),
         }
+        except KeyError:
+            logging.debug("InfoCache.parse_file: FileNotLessonfile(%s)" % filename)
+            raise self.FileNotLessonfile(filename)
     def iter_parse_all_files(self):
         """
         Parse the files and put then in the cache.
