@@ -93,6 +93,8 @@ class MidiEventStream(object):
         Bad name, but I don't have a better idea right now. This
         class will handle all 16 midi channels.
         """
+        # We are zero-indexed, so this is MIDI channel 10
+        percussion_MIDI_channel = 9
         class MidiChannel(object):
             def __init__(self, number):
                 self.m_number = number
@@ -108,15 +110,18 @@ class MidiEventStream(object):
                 Return True if no tones are playing on this channel.
                 """
                 return not bool(self.m_tones)
-        def __init__(self):
-            # We are zero-indexed, so this is MIDI channel 10
-            self.percussion_MIDI_channel = 9
+        def __init__(self, nc=16):
             self.free_MIDI_channels = []
-            for i in range(self.percussion_MIDI_channel) \
-                     + range(self.percussion_MIDI_channel, 16):
-                self.free_MIDI_channels.append(self.MidiChannel(i))
+            if nc > self.percussion_MIDI_channel:
+                for i in range(self.percussion_MIDI_channel) \
+                         + range(self.percussion_MIDI_channel, 16):
+                    self.free_MIDI_channels.append(self.MidiChannel(i))
+            else:
+                for i in range(nc):
+                    self.free_MIDI_channels.append(self.MidiChannel(i))
             # The dict key will be the (patch number, volume).
-            # The value is a MidiChannel object.
+            # The value is a list of MidiChannel object that play with
+            # the patch and volume the key say.
             self.allocated_MIDI_channels = {}
             # This dict maps from MIDI channel number to the actual
             # MidiChannel object
@@ -127,23 +132,37 @@ class MidiEventStream(object):
             setattr(self.int_to_channel_object[channel], key, data)
         def get_channel_data(self, channel, key):
             return getattr(self.int_to_channel_object[channel], key, None)
-        def alloc_channel(self, patch, volume):
-            """
-            Allocate a midi channel for the patch and
-            return the MIDI channel number of the allocated channel.
-            """
-            #FIXME need to handle running out of available midi channels.
+        def require_channel(self, pitch, patch, volume):
             key = (patch, volume)
-            assert key not in self.allocated_MIDI_channels
+            if key in self.allocated_MIDI_channels:
+                for channel in self.allocated_MIDI_channels[key]:
+                    if pitch not in channel.m_tones:
+                        return channel.m_number
+                else:
+                    # we have to alloc a new MIDI channel if the pitch is
+                    # already playing
+                    self.allocated_MIDI_channels[key].append(self.alloc_channel())
+                    return self.allocated_MIDI_channels[key][-1].m_number
+            else:
+                self.allocated_MIDI_channels[key] = [self.alloc_channel()]
+                return self.allocated_MIDI_channels[key][-1].m_number
+            return -1
+        def alloc_channel(self):
+            """
+            Return a unused MIDI channel. Search for silent channels first,
+            allocate a new if none found.
+            """
+            if self.free_MIDI_channels:
+                return self.free_MIDI_channels.pop(0)
             # First try to find an allocated channel that is silent.
-            for k, channel in self.allocated_MIDI_channels.items():
-                if channel.is_silent():
-                    self.allocated_MIDI_channels[key] = \
-                        self.allocated_MIDI_channels[k]
-                    del self.allocated_MIDI_channels[k]
-                    return self.allocated_MIDI_channels[key].m_number
-            self.allocated_MIDI_channels[key] = self.free_MIDI_channels.pop(0)
-            return self.allocated_MIDI_channels[key].m_number
+            for key, channel_list in self.allocated_MIDI_channels.items():
+                for idx, channel in enumerate(channel_list):
+                    if channel.is_silent():
+                        ret = self.allocated_MIDI_channels[key].pop(idx)
+                        if not self.allocated_MIDI_channels[key]:
+                            del self.allocated_MIDI_channels[key]
+                        return ret
+            raise Exception("FIXME: handle running out of MIDI channels!")
         def get_channel_for_patch(self, patch, volume):
             """
             Return the MIDI channel number we should use to play a tone
@@ -158,6 +177,9 @@ class MidiEventStream(object):
         def is_playing(self, channel, pitch):
             return pitch in self.int_to_channel_object[channel].m_tones
     def __init__(self, *tracks):
+        # The number of MIDI channels are changed in the test suite to easier
+        # test handling of too few midi channels.
+        self.num_MIDI_channels = 16
         self.m_tracks = tracks
         for track in self.m_tracks:
             track.calculate_event_times()
@@ -204,8 +226,22 @@ class MidiEventStream(object):
                 data[e[0]].append(e)
         for event in data[self.NOTE_OFF]:
             yield event
-
     def __iter__(self):
+        ret = []
+        for e in self.__mkevents():
+            if e[0] == 'program-change' and ret:
+                i = len(ret)
+                while (isinstance(ret[i-1][1], Rat) or ret[i-1][1] != e[1]) and i > 1:
+                    i -= 1
+                if i < len(ret):
+                    while ret[i][0] == 'program-change' and i < len(ret) - 1 and ret[i][1] < e[1]:
+                        i += 1
+                ret.insert(i, e)
+            else:
+                ret.append(e)
+        for e in ret:
+            yield e
+    def __mkevents(self):
         # tpos_set will know all the positions in time where anything happens
         # on any staff
         tpos_set = set()
@@ -222,13 +258,13 @@ class MidiEventStream(object):
                         'patch-requested': 0}
         tempo_request = None
         tempo_current = None
-        # We use this list of dicts to know which tones are playing
-        # which patch on the tracks. The key of the dicts are the integer
-        # value representing the pitch, and the value is the patch number.
+        # We use this list of dicts to know which MIDI channel the tones are
+        # playing on. The key of the dicts are the integer value representing
+        # the pitch, and the value is the MIDI channel the tone is playing on.
         track_notes = []
         for x in range(len(self.m_tracks)):
             track_notes.append({})
-        ch_dev = self.ChannelDevice()
+        ch_dev = self.ChannelDevice(self.num_MIDI_channels)
         last_pos = Rat(0, 1)
         for tpos in tpos_list:
             if tpos != last_pos: # Just to not insert before the first events
@@ -241,15 +277,11 @@ class MidiEventStream(object):
                             # with the wrong pitch.
                             logging.debug("not stopping, not playing now: %s" % e)
                             continue
-                        patch = track_notes[idx][e.m_pitch]
+                        chn = track_notes[idx][e.m_pitch]
                         del track_notes[idx][e.m_pitch]
-                        if isinstance(self.m_tracks[idx], PercussionTrack):
-                            chn = ch_dev.percussion_MIDI_channel
-                        else:
-                            chn = ch_dev.get_channel_for_patch(patch, track_state[idx]['volume-requested'])
-                        if ch_dev.is_playing(chn, e.m_pitch):
-                            ch_dev.stop_note(chn, e.m_pitch)
-                            yield self.NOTE_OFF, chn, e.m_pitch, e.m_velocity
+                        assert ch_dev.is_playing(chn, e.m_pitch)
+                        ch_dev.stop_note(chn, e.m_pitch)
+                        yield self.NOTE_OFF, chn, e.m_pitch, e.m_velocity
             for idx, track in enumerate(tracks2):
                 if tpos in track:
                     for e in track[tpos]['OtherEvents']:
@@ -274,12 +306,7 @@ class MidiEventStream(object):
                                 ch_dev.set_channel_data(chn, 'volume', track_state[idx]['volume-requested'])
                                 yield self.VOLUME, chn, track_state[idx]['volume-requested']
                         else:
-                            try:
-                                chn = ch_dev.get_channel_for_patch(
-                                        track_state[idx]['patch-requested'],
-                                        track_state[idx]['volume-requested'])
-                            except KeyError:
-                                chn = ch_dev.alloc_channel(
+                            chn = ch_dev.require_channel(e.m_pitch,
                                         track_state[idx]['patch-requested'],
                                         track_state[idx]['volume-requested'])
                             if ch_dev.get_channel_data(chn, 'patch') != track_state[idx]['patch-requested']:
@@ -291,7 +318,7 @@ class MidiEventStream(object):
                         if ch_dev.is_playing(chn, e.m_pitch):
                             logging.debug("MidiEventStream: ignoring duplicate tone: %s" % e)
                             continue
-                        track_notes[idx][e.m_pitch] = track_state[idx]['patch-requested']
+                        track_notes[idx][e.m_pitch] = chn
                         # ch_dev must know which tones are sounding on which
                         # MIDI channels, so it can handle the midi resources.
                         ch_dev.start_note(chn, e.m_pitch)
