@@ -887,6 +887,8 @@ class TestLabelObject(unittest.TestCase):
         for s, v in (
             ("Imaj7-", [('I', 'maj', '7', '-')]),
             ("bIIIm7 -", [('bIII', 'm', '7', ' -')]),
+            (u"♯IIIm7 -", [(u'♯III', 'm', '7', ' -')]),
+            (u"♭IIIm7 -", [(u'♭III', 'm', '7', ' -')]),
             ("Imaj7-IIm7-V9-Imaj7", [('I', 'maj', '7', '-'),
                                      ('II', 'm', '7', '-'),
                                      ('V', '', '9', '-'),
